@@ -1,6 +1,4 @@
-#' @name create_batch_glossary
-#' @export create_batch_glossary
-#' @title Creates a [base::data.frame()] that help batching long-running read and writes.
+#' @title Creates a [base::data.frame()] that help batching long-running read and writes
 #'
 #' @description The function returns a [base::data.frame()] that other functions use to separate long-running
 #' read and write REDCap calls into multiple, smaller REDCap calls.  The goal is to (1) reduce the chance of time-outs,
@@ -28,27 +26,25 @@
 #' See [redcap_read()] for a function that uses `create_batch_gloassary`.
 #'
 #' @examples
-#' library(REDCapR) #Load the package into the current R session.
-#' create_batch_glossary(100, 50)
-#' create_batch_glossary(100, 25)
-#' create_batch_glossary(100, 3)
+#' REDCapR::create_batch_glossary(100, 50)
+#' REDCapR::create_batch_glossary(100, 25)
+#' REDCapR::create_batch_glossary(100, 3)
 #' d <- data.frame(
 #'   record_id = 1:100,
 #'   iv        = sample(x=4, size=100, replace=TRUE),
 #'   dv        = rnorm(n=100)
 #' )
-#' create_batch_glossary(nrow(d), batch_size=40)
+#' REDCapR::create_batch_glossary(nrow(d), batch_size=40)
 
+#' @export
 create_batch_glossary <- function( row_count, batch_size ) {
-  if( !(!is.na(row_count) & (length(row_count)==1L) & (row_count[1]>=1) | inherits(row_count, "integer")) )
-    stop("`row_count` must be a positive scalar integer.")
-  if( !(!is.na(batch_size) & (length(batch_size)==1L) & (batch_size[1]>=1) | inherits(row_count, "integer")) )
-    stop("`batch_size` must be a positive scalar integer.")
+  checkmate::assert_integerish(row_count , any.missing=F, len=1L, lower=1L)
+  checkmate::assert_integerish(batch_size, any.missing=F, len=1L, lower=1L)
 
   start_index <- base::seq.int(from=1, to=row_count, by=batch_size)
 
   ds_batch             <- base::data.frame(id=seq_along(start_index), start_index=start_index)
-  ds_batch$stop_index  <- base::mapply(function(i) base::ifelse(i<length(start_index), start_index[i+1]-1, row_count), ds_batch$id )
+  ds_batch$stop_index  <- base::mapply(function(i) base::ifelse(i<length(start_index), start_index[i+1L]-1L, row_count), ds_batch$id )
 
   sprintf_format_1 <- base::paste0("%0.", base::max(base::nchar(ds_batch$id)), "i")
   sprintf_format_2 <- base::paste0("%0.", base::nchar(row_count), "i")
