@@ -1,6 +1,4 @@
-#' @name redcap_read
-#' @export redcap_read
-#' @title Read records from a REDCap project in subsets, and stacks them together before returning a [base::data.frame()].
+#' @title Read records from a REDCap project in subsets, and stacks them together before returning a [base::data.frame()]
 #'
 #' @description From an external perspective, this function is similar to [redcap_read_oneshot()].  The internals
 #' differ in that `redcap_read` retrieves subsets of the data, and then combines them before returning
@@ -14,18 +12,27 @@
 #' @param token The user-specific string that serves as the password for a project.  Required.
 #' @param records An array, where each element corresponds to the ID of a desired record.  Optional.
 #' @param records_collapsed A single string, where the desired ID values are separated by commas.  Optional.
-#' @param fields An array, where each element corresponds a desired project field.  Optional.
+#' @param fields An array, where each element corresponds to a desired project field.  Optional.
 #' @param fields_collapsed A single string, where the desired field names are separated by commas.  Optional.
-#' @param filter_logic String of logic text (e.g., `[gender] = 'male'`) for filtering the data to be returned by this API method, in which the API will only return the records (or record-events, if a longitudinal project) where the logic evaluates as TRUE.   An blank/empty string returns all records.
-#' @param events An array, where each element corresponds a desired project event  Optional.
+#' @param forms An array, where each element corresponds to a desired project form.  Optional.
+#' @param forms_collapsed A single string, where the desired form names are separated by commas.  Optional.
+#' @param events An array, where each element corresponds to a desired project event.  Optional.
 #' @param events_collapsed A single string, where the desired event names are separated by commas.  Optional.
-#' @param export_data_access_groups A boolean value that specifies whether or not to export the `redcap_data_access_group` field when data access groups are utilized in the project. Default is `FALSE`. See the details below.
 #' @param raw_or_label A string (either 'raw` or 'label' that specifies whether to export the raw coded values or the labels for the options of multiple choice fields.  Default is `'raw'`.
+#' @param raw_or_label_headers A string (either `'raw'` or `'label'` that specifies for the CSV headers whether to export the variable/field names (raw) or the field labels (label).  Default is `'raw'`.
+#' @param export_checkbox_label specifies the format of checkbox field values specifically when exporting the data as labels.  If `raw_or_label` is `'label'` and `export_checkbox_label` is TRUE, the values will be the text displayed to the users.  Otherwise, the values will be 0/1.
+# placeholder: returnFormat
+#' @param export_survey_fields A boolean that specifies whether to export the survey identifier field (e.g., 'redcap_survey_identifier') or survey timestamp fields (e.g., instrument+'_timestamp') .
+#' @param export_data_access_groups A boolean value that specifies whether or not to export the `redcap_data_access_group` field when data access groups are utilized in the project. Default is `FALSE`. See the details below.
+#' @param filter_logic String of logic text (e.g., `[gender] = 'male'`) for filtering the data to be returned by this API method, in which the API will only return the records (or record-events, if a longitudinal project) where the logic evaluates as TRUE.   An blank/empty string returns all records.
+#'
+#' @param guess_type A boolean value indicating if all columns should be returned as character.  If true, [readr::read_csv()] guesses the intended data type for each column.
+#' @param guess_max A positive integer passed to [readr::read_csv()] **per batch** that specifies the maximum number of records to use for guessing column types.
 #' @param verbose A boolean value indicating if `message`s should be printed to the R console during the operation.  The verbose output might contain sensitive information (*e.g.* PHI), so turn this off if the output might be visible somewhere public. Optional.
 #' @param config_options  A list of options to pass to `POST` method in the `httr` package.  See the details in `redcap_read_oneshot()` Optional.
 #' @param id_position  The column position of the variable that unique identifies the subject.  This defaults to the first variable in the dataset.
 #'
-#' @return Currently, a list is returned with the following elements,
+#' @return Currently, a list is returned with the following elements:
 #' * `data`: An R [base::data.frame()] of the desired records and columns.
 #' * `success`: A boolean value indicating if the operation was apparently successful.
 #' * `status_codes`: A collection of [http status codes](http://en.wikipedia.org/wiki/List_of_HTTP_status_codes), separated by semicolons.  There is one code for each batch attempted.
@@ -34,6 +41,7 @@
 #' * `fields_collapsed`: The desired field names, collapsed into a single string, separated by commas.
 #' * `filter_logic`: The filter statement passed as an argument.
 #' * `elapsed_seconds`: The duration of the function.
+#'
 #' @details
 #' Specifically, it internally uses multiple calls to [redcap_read_oneshot()] to select and return data.
 #' Initially, only primary key is queried through the REDCap API.  The long list is then subsetted into partitions,
@@ -51,47 +59,77 @@
 #'
 #' @author Will Beasley
 #' @references The official documentation can be found on the 'API Help Page' and 'API Examples' pages
-#' on the REDCap wiki (ie, https://community.projectredcap.org/articles/456/api-documentation.html and
+#' on the REDCap wiki (*i.e.*, https://community.projectredcap.org/articles/456/api-documentation.html and
 #' https://community.projectredcap.org/articles/462/api-examples.html). If you do not have an account
 #' for the wiki, please ask your campus REDCap administrator to send you the static material.
 #'
 #' @examples
 #' \dontrun{
-#' library(REDCapR) #Load the package into the current R session.
 #' uri     <- "https://bbmc.ouhsc.edu/redcap/api/"
 #' token   <- "9A81268476645C4E5F03428B8AC3AA7B"
-#' redcap_read(batch_size=2, redcap_uri=uri, token=token)
+#' REDCapR::redcap_read(batch_size=2, redcap_uri=uri, token=token)
 #' }
-#'
 
+#' @export
 redcap_read <- function(
-  batch_size=100L, interbatch_delay=0.5, continue_on_error=FALSE,
-  redcap_uri, token, records=NULL, records_collapsed="",
-  fields=NULL, fields_collapsed="",
-  events=NULL, events_collapsed="",
-  export_data_access_groups=FALSE,
-  filter_logic="",
-  raw_or_label='raw',
-  verbose=TRUE, config_options=NULL, id_position=1L
+  batch_size                    = 100L,
+  interbatch_delay              = 0.5,
+  continue_on_error             = FALSE,
+  redcap_uri,
+  token,
+  records                       = NULL, records_collapsed = "",
+  fields                        = NULL, fields_collapsed  = "",
+  forms                         = NULL, forms_collapsed   = "",
+  events                        = NULL, events_collapsed  = "",
+  raw_or_label                  = "raw",
+  raw_or_label_headers          = "raw",
+  export_checkbox_label         = FALSE,
+  # placeholder: returnFormat
+  export_survey_fields          = FALSE,
+  export_data_access_groups     = FALSE,
+  filter_logic                  = "",
+
+  guess_type                    = TRUE,
+  guess_max                     = 1000L,
+  verbose                       = TRUE,
+  config_options                = NULL,
+  id_position                   = 1L
 ) {
 
-  if( missing(redcap_uri) )
-    stop("The required parameter `redcap_uri` was missing from the call to `redcap_read()`.")
+  checkmate::assert_character(redcap_uri                , any.missing=F, len=1, pattern="^.{1,}$")
+  checkmate::assert_character(token                     , any.missing=F, len=1, pattern="^.{1,}$")
+  checkmate::assert_atomic(  records                    , any.missing=T, min.len=0)
+  checkmate::assert_character(records_collapsed         , any.missing=T, len=1, pattern="^.{0,}$", null.ok=T)
+  checkmate::assert_character(fields                    , any.missing=T, min.len=1, pattern="^.{1,}$", null.ok=T)
+  checkmate::assert_character(fields_collapsed          , any.missing=T, len=1, pattern="^.{0,}$", null.ok=T)
+  checkmate::assert_character(forms                     , any.missing=T, min.len=1, pattern="^.{1,}$", null.ok=T)
+  checkmate::assert_character(forms_collapsed           , any.missing=T, len=1, pattern="^.{0,}$", null.ok=T)
+  checkmate::assert_character(events                    , any.missing=T, min.len=1, pattern="^.{1,}$", null.ok=T)
+  checkmate::assert_character(events_collapsed          , any.missing=T, len=1, pattern="^.{0,}$", null.ok=T)
+  checkmate::assert_character(raw_or_label              , any.missing=F, len=1)
+  checkmate::assert_subset(   raw_or_label              , c("raw", "label"))
+  checkmate::assert_character(raw_or_label_headers      , any.missing=F, len=1)
+  checkmate::assert_subset(   raw_or_label_headers      , c("raw", "label"))
+  checkmate::assert_logical(  export_checkbox_label     , any.missing=F, len=1)
+  # placeholder: returnFormat
+  checkmate::assert_logical(  export_survey_fields      , any.missing=F, len=1)
+  checkmate::assert_logical(  export_data_access_groups , any.missing=F, len=1)
+  #
+  checkmate::assert_logical(  guess_type                , any.missing=F, len=1)
+  checkmate::assert_integerish(guess_max                , any.missing=F, len=1, lower=1)
+  checkmate::assert_logical(  verbose                   , any.missing=F, len=1, null.ok=T)
+  checkmate::assert_list(     config_options            , any.missing=T, len=1, null.ok=T)
+  checkmate::assert_integer(  id_position               , any.missing=F, len=1, lower=1L)
 
-  if( missing(token) )
-    stop("The required parameter `token` was missing from the call to `redcap_read()`.")
-
-  token <- sanitize_token(token)
   validate_field_names(fields)
 
-  if( all(nchar(records_collapsed)==0) )
-    records_collapsed <- ifelse(is.null(records), "", paste0(records, collapse=",")) #This is an empty string if `records` is NULL.
-  if( (length(fields_collapsed)==0L) | is.null(fields_collapsed) | all(nchar(fields_collapsed)==0) )
-    fields_collapsed <- ifelse(is.null(fields), "", paste0(fields, collapse=",")) #This is an empty string if `fields` is NULL.
-  if( all(nchar(events_collapsed)==0) )
-    events_collapsed <- ifelse(is.null(events), "", paste0(events, collapse=",")) #This is an empty string if `events` is NULL.
-  if( all(nchar(filter_logic)==0) )
-    filter_logic <- ifelse(is.null(filter_logic), "", filter_logic) #This is an empty string if `filter_logic` is NULL.
+  token               <- sanitize_token(token)
+  records_collapsed   <- collapse_vector(records  , records_collapsed)
+  fields_collapsed    <- collapse_vector(fields   , fields_collapsed)
+  forms_collapsed     <- collapse_vector(forms    , forms_collapsed)
+  events_collapsed    <- collapse_vector(events   , events_collapsed)
+  filter_logic        <- filter_logic_prepare(filter_logic)
+  verbose             <- verbose_prepare(verbose)
 
   start_time <- Sys.time()
 
@@ -106,23 +144,26 @@ redcap_read <- function(
     redcap_uri         = redcap_uri,
     token              = token,
     records_collapsed  = records_collapsed,
-    fields_collapsed   = metadata$data[1, "field_name"],
-    filter_logic       = filter_logic,
+    fields_collapsed   = metadata$data$field_name[id_position],
+    forms_collapsed    = forms_collapsed,
     events_collapsed   = events_collapsed,
+    filter_logic       = filter_logic,
+    guess_type         = guess_type,
     verbose            = verbose,
     config_options     = config_options
   )
 
   # Stop and return to the caller if the initial query failed. --------------
   if( !initial_call$success ) {
-    outcome_messages <- paste0("The initial call failed with the code: ", initial_call$status_code, ".")
-    elapsed_seconds <- as.numeric(difftime(Sys.time(), start_time, units="secs"))
+    outcome_messages  <- paste0("The initial call failed with the code: ", initial_call$status_code, ".")
+    elapsed_seconds   <- as.numeric(difftime(Sys.time(), start_time, units="secs"))
     return( list(
       data                  = data.frame(),
       records_collapsed     = "failed in initial batch call",
       fields_collapsed      = "failed in initial batch call",
-      filter_logic          = "failed in initial batch call",
+      forms_collapsed       = "failed in initial batch call",
       events_collapsed      = "failed in initial batch call",
+      filter_logic          = "failed in initial batch call",
       elapsed_seconds       = elapsed_seconds,
       status_code           = initial_call$status_code,
       outcome_messages      = outcome_messages,
@@ -131,19 +172,10 @@ redcap_read <- function(
   }
 
   # Continue as intended if the initial query succeeded. --------------------
-  uniqueIDs <- sort(unique(initial_call$data[, 1]))
+  uniqueIDs <- sort(unique(initial_call$data[[id_position]]))
 
   if( all(nchar(uniqueIDs)==32L) )
-    warning(
-      "It appears that the REDCap record IDs have been hashed. ",
-      "For `redcap_read` to function properly, the user must have Export permissions for the 'Full Data Set'. ",
-      "To grant the appropriate permissions: ",
-      "(1) go to 'User Rights' in the REDCap project site, ",
-      "(2) select the desired user, and then select 'Edit User Privileges', ",
-      "(3) in the 'Data Exports' radio buttons, select 'Full Data Set'.\n",
-      "Users with only `De-Identified` export privileges can still use ",
-      "`redcap_read_oneshot()` and `redcap_write_oneshot()`."
-    )
+    warn_hash_record_id()
 
   ds_glossary            <- REDCapR::create_batch_glossary(row_count=length(uniqueIDs), batch_size=batch_size)
   lst_batch              <- NULL
@@ -152,10 +184,10 @@ redcap_read <- function(
   lst_outcome_message    <- NULL
   success_combined       <- TRUE
 
-  message("Starting to read ", format(length(uniqueIDs), big.mark=",", scientific=F, trim=T), " records  at ", Sys.time())
+  message("Starting to read ", format(length(uniqueIDs), big.mark=",", scientific=F, trim=T), " records  at ", Sys.time(), ".")
   for( i in ds_glossary$id ) {
-    selected_index <- seq(from=ds_glossary[i, "start_index"], to=ds_glossary[i, "stop_index"])
-    selected_ids <- uniqueIDs[selected_index]
+    selected_index  <- seq(from=ds_glossary$start_index[i], to=ds_glossary$stop_index[i])
+    selected_ids    <- uniqueIDs[selected_index]
 
     if( i > 0 ) Sys.sleep(time = interbatch_delay)
     if( verbose ) {
@@ -169,17 +201,24 @@ redcap_read <- function(
       token                       = token,
       records                     = selected_ids,
       fields_collapsed            = fields_collapsed,
-      filter_logic                = filter_logic,
       events_collapsed            = events_collapsed,
-      export_data_access_groups   = export_data_access_groups,
+      forms_collapsed             = forms_collapsed,
       raw_or_label                = raw_or_label,
+      raw_or_label_headers        = raw_or_label_headers,
+      export_checkbox_label       = export_checkbox_label,
+      # placeholder: return_format
+      export_survey_fields        = export_survey_fields,
+      export_data_access_groups   = export_data_access_groups,
+      filter_logic                = filter_logic,
+
+      guess_type                  = guess_type,
+      guess_max                   = guess_max,
       verbose                     = verbose,
       config_options              = config_options
     )
 
-    lst_status_code[[i]] <- read_result$status_code
-    # lst_status_message[[i]] <- read_result$status_message
-    lst_outcome_message[[i]] <- read_result$outcome_message
+    lst_status_code[[i]]      <- read_result$status_code
+    lst_outcome_message[[i]]  <- read_result$outcome_message
 
     if( !read_result$success ) {
       error_message <- paste0("The `redcap_read()` call failed on iteration ", i, ".")
@@ -194,12 +233,12 @@ redcap_read <- function(
 
     rm(read_result) #Admittedly overkill defensiveness.
   }
-  # browser()
-  ds_stacked               <- as.data.frame(data.table::rbindlist(lst_batch))
-  # ds_stacked               <- as.data.frame(dplyr::bind_rows(lst_batch))
+
+  # ds_stacked               <- as.data.frame(data.table::rbindlist(lst_batch))
+  ds_stacked               <- as.data.frame(dplyr::bind_rows(lst_batch))
 
   elapsed_seconds          <- as.numeric(difftime( Sys.time(), start_time, units="secs"))
-  status_code_combined     <- paste(lst_status_code, collapse="; ")
+  status_code_combined     <- paste(lst_status_code    , collapse="; ")
   outcome_message_combined <- paste(lst_outcome_message, collapse="; ")
 
   return( list(
@@ -209,8 +248,9 @@ redcap_read <- function(
     outcome_messages    = outcome_message_combined,
     records_collapsed   = records_collapsed,
     fields_collapsed    = fields_collapsed,
-    filter_logic        = filter_logic,
+    forms_collapsed     = forms_collapsed,
     events_collapsed    = events_collapsed,
+    filter_logic        = filter_logic,
     elapsed_seconds     = elapsed_seconds
   ) )
 }
@@ -218,3 +258,16 @@ redcap_read <- function(
 # redcap_uri <- "https://bbmc.ouhsc.edu/redcap/api/"
 # token <- "9A81268476645C4E5F03428B8AC3AA7B"
 # redcap_read(batch_size=2, redcap_uri=redcap_uri, token=token)
+
+warn_hash_record_id <- function( )  {
+  warning(
+    "It appears that the REDCap record IDs have been hashed.\n",
+    "For `REDCapR::redcap_read()` to function properly, the user must have Export permissions for the 'Full Data Set'.\n",
+    "To grant the appropriate permissions:\n",
+    "(1) go to 'User Rights' in the REDCap project site,\n",
+    "(2) select the desired user, and then select 'Edit User Privileges',\n",
+    "(3) in the 'Data Exports' radio buttons, select 'Full Data Set'.\n",
+    "Users with only `De-Identified` export privileges can still use\n",
+    "`redcap_read_oneshot()` and `redcap_write_oneshot()`."
+  )
+}
