@@ -2,7 +2,7 @@ library(testthat)
 context("Version")
 
 credential <- REDCapR::retrieve_credential_local(
-  path_credential = base::file.path(devtools::inst(name="REDCapR"), "misc/example.credentials"),
+  path_credential = system.file("misc/example.credentials", package="REDCapR"),
   project_id      = 153
 )
 
@@ -13,17 +13,17 @@ test_that("Smoke Test", {
   )
 })
 
-test_that("Version Successfull", {
+test_that("Version Successful", {
   testthat::skip_on_cran()
   expect_message(
     actual <- redcap_version(redcap_uri=credential$redcap_uri, token=credential$token)
   )
 
-  expected <- package_version("7.3.2")
+  expected <- package_version("9.0.0")
   expect_equal(actual, expected)
 })
 
-test_that("Version Successfull", {
+test_that("Version Unuccessful --bad token", {
   testthat::skip_on_cran()
   expect_message(
     actual <- redcap_version(redcap_uri=credential$redcap_uri, token="0000008476645C4E5F03428B8AC3AA7C")
